
import libxml2
import libxslt

from tempfile import mkstemp
from cStringIO import StringIO

import os
import string
import re
import pickle
from subprocess import Popen,PIPE,STDOUT
import ConfigParser
from time import sleep
import sys
import signal

from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from xml.sax.saxutils import escape

from Config import Configuration

class XMLHandler(ContentHandler):
    def __init__(self,fields):
        ContentHandler.__init__(self)
        self.record = False
        self.initialize=dict()
        for field in fields:
            self.initialize[field]=""
        self.gamelist=list()

    def startElement(self, name, attrs):
        if name == "game":
            game=self.initialize.copy()
            for value in attrs.getNames():
                game[value]=attrs.get(value)
            self.gamelist.append(game)
        
    def get_list(self):
        return self.gamelist

class Loader:
    def __init__(self):
        return

    def load(self):
        self.config.load()
        file = self.config.get("General","gamelist")
        if not os.access(file,os.R_OK):
             return _("File %s not found") % file
        format = self.config.get("General","gamelist_format")
        if format == "xml":
            handler = XMLHandler(self.config.get("General","fields"))
            saxparser = make_parser()
            saxparser.setContentHandler(handler)
            saxparser.parse(open(file))
            self.list = handler.get_list()
        elif format == "python":
            self.list = pickle.load(open(file))
        else:
            return _("Unknown format: %s") % format
        return None

class Manager(Loader):
    def __init__(self):
        Loader.__init__(self)
        self.config = Configuration()
        self.config.load()
        self.pattern=dict()
        self.list=list()
        self.match=list()

    def get_filter_lists(self):
        lists=dict()
        shown_states=self.config.get("General","shown_states")
        selects=self.config.get("General","selects")
        for name in selects:
            lists[name]=list()
        for game in self.list:
            if game["state"] in shown_states:
                for name in selects:
                    if game[name] not in lists[name]:
                        lists[name].append(game[name])
        for name in selects:
            lists[name].sort()
            lists[name].insert(0,_("All"))
        return lists

    def set_selection(self,category,selection):
        self.pattern=dict()
        if _('All') not in selection:
            self.pattern[category]=("^"+string.join(selection,"$|^")+"$").replace('*','\*')
        self.pattern['emulator']=self.emu
        return

    def set_emulator(self,emulator):
        self.emu="^"+emulator+"$"
        self.pattern['emulator']=self.emu
        return

    def filter(self,pattern=None):
        if pattern:
            self.pattern=pattern
            self.pattern['emulator']=self.emu
        self.match=self.list
        if 'state' not in self.pattern and self.config.has_option('General','shown_states'):
            shown_states=self.config.get('General','shown_states')
            self.pattern['state']=("^"+string.join(shown_states,"$|^")+"$").replace('*','\*')
        for name,value in self.pattern.iteritems():
            if value != "" and value != "^"+_('All')+"$" :
                self.match=self.match_pattern(self.match,name,value)
        return self.match

    def match_pattern(self,match_list,name,pattern):
        rx=re.compile(pattern,re.I)
        match=list()
        for game in match_list:
            if rx.search(game[name]):
                match.append(game)
        return match

    def get(self,index):
        return self.match[index]
        
    def execute(self,index):
        id=self.match[index]["emulator"]
        emu=self.config.emu(id)
        
        ident= emu.get("General","identity")
        args=list() 
        for arg in emu.get("General","start").split(" "):
            if arg != '':
                args.append(arg.replace("@(identity)",self.match[index][ident]))
        p=Popen(args,close_fds=True,stderr=STDOUT,stdout=PIPE)
        return p

class Builder(Loader):
    def __init__(self):
        Loader.__init__(self)
        self.config = Configuration()
        self.config.load()
        if not os.access(os.path.dirname(self.config.get("General","gamelist")),os.W_OK):
            self.log("99",_("Can not write gamelist file : %s") % self.config.get("General","gamelist"))
            sys.exit(1)
            return
        self.out= sys.stdout
        signal.signal(signal.SIGTERM,self.handler)

    def build(self):
        self.list=list()
        for id in self.config.emulators():
            self.build_id(id)
            self.list = self.list + self.current.values()
        self.write()

    def build_emulator(self,id):
        if not os.access(self.config.get("General","gamelist"),os.R_OK):
             self.list=list()
        else:
            self.log("10",_("Loading gamelist ..."))
            self.log("11",self.config.get("General","gamelist"))
            self.log("00",_(" "))
            err=self.load()
            if err:
                self.log("99",err)
        self.id=id
        self.list=filter(self.remove_id,self.list)
        self.build_id(id)
        self.list = self.list + self.current.values()
        self.write()

    def remove_id(self,game):
        if game['emulator'] == self.id:
            return False
        return True

    def build_id(self,id):
        self.current=dict()
        emu=self.config.emu(id)

        for path in os.environ['PATH'].split(":"):
             if os.access(path + "/" + emu.get("General","binary"),os.X_OK):
                break
        else:
            return

        for section in emu.get("General","sources"):
            source=emu.get(section,"source")
            type=emu.get(section,"type")
            format = emu.get(section,"format")

            if type=="archive":
                src = None
                if os.access(source,os.R_OK):
                    src = open(source)
                self.log("10",_("Loading %s file ...") % format)
                self.log("11",source)
            elif type=="command":
                p=Popen(source.split(" "),close_fds=True,stdout=PIPE,stderr=PIPE)
                src = p.stdout
                self.log("10",_("Loading %s output ...") % format)
                self.log("11",source)
            else:
                self.log("99",_("Type of source unknown ['file','command'] : %s") % type)
                sys.exit(1)
                return
            
            if format == "ini":
                self.load_ini_source(src,emu,section)
            elif format == "xml":
                self.load_xml_source(src,emu,section)
            elif format == "plain":
                self.load_plain_source(src,emu,section)
            else:
                self.log("99",_("Format of source unknown ['xml','ini','plain'] : %s") % format)
                sys.exit(1)
                return

        transform_values=dict()
        for field in self.config.get('General','fields'):
            if field in emu.options('General'):
                transform_values[field]=emu.get('General',field)
        
        for game in self.current:
            self.current[game]["emulator"]=id
            for key,value in self.current[game].iteritems():
                if key in transform_values:
                    self.current[game][key]=transform_values[key].get(value,value)
        return

    def load_ini_source(self,src,emu,section):
        if src == None:
            return
        self.log("00",_("Initializing ..."))
        ident=emu.get("General","identity")
        if emu.has_option(section,"sections"):
            self.load_ini_sections(src,ident,emu.get(section,"sections"))
        elif emu.has_option(section,"options"):
            self.load_ini_options(src,ident,emu.get(section,"options"))

    def load_ini_sections(self,src,ilabel,sections):
        parser= ConfigParser.RawConfigParser()
        parser.readfp(src)
        for title,key in sections.iteritems():
            self.log("00",_("Parsing %s section ...") % title)
            if parser.has_section(title):
                for ident,value in parser.items(title):
                    if not self.current.has_key(ident):
                        self.current[ident]=dict()
                        for field in self.config.get("General","fields"):
                            self.current[ident][field]=""
                        self.current[ident][ilabel]=ident
                    self.current[ident][key]=value
        return 

    def load_ini_options(self,src,ilabel,options):
        parser= ConfigParser.RawConfigParser()
        parser.readfp(src)
        for ident in parser.sections():
            self.log("00",_("Parsing %s section ...") % ident)
            for title,value in parser.items(ident):
                if options.has_key(title):
                    if not self.current.has_key(ident):
                        self.current[ident]=dict()
                        for field in self.config.get("General","fields"):
                            self.current[ident][field]=""
                        self.current[ident][ilabel]=ident
                    self.current[ident][options[title]]=value
        return

    def load_xml_source(self,src,emu,section):
        if src == None:
            return
        self.log("00",_("Initializing ..."))
        if os.access(emu.get(section,"xsl"),os.R_OK):
  	        xslfile=emu.get(section,"xsl")
        elif os.access(self.config.config_dir + emu.get(section,"xsl"),os.R_OK):
  	        xslfile=self.config.config_dir + emu.get(section,"xsl")
        else:
            return
        tempfile=mkstemp()
        os.write(tempfile[0],src.read())

        self.log("00",_("Parsing XSL file (%s)...") % xslfile)
        styledoc = libxml2.parseFile(xslfile)
        style = libxslt.parseStylesheetDoc(styledoc)
        self.log("00",_("Parsing xml output ..."))
        doc = libxml2.parseFile(tempfile[1])
        self.log("00",_("Applying XSL (%s) ...") % xslfile)
        result = style.applyStylesheet(doc, None)

        self.log("00",_("Saving temporal result ..."))
        str= style.saveResultToString(result)
        style.freeStylesheet()
        doc.freeDoc()
        result.freeDoc()

        self.log("00","Initializing result ...")
        sio=StringIO(str)
        handler = XMLHandler(self.config.get("General","fields"))
        saxparser = make_parser()
        saxparser.setContentHandler(handler)
        self.log("00","Parsing result ...")
        saxparser.parse(sio)
        
        self.log("00","Finalizing ...")
        ilabel=emu.get("General","identity")
        for game in handler.get_list():
            ident=game[ilabel].strip()
            if not self.current.has_key(ident):
                self.current[ident]=dict()
                for field in self.config.get("General","fields"):
                    self.current[ident][field]=""
            for key,value in game.iteritems():
                if self.current[ident][key] == "":
                    self.current[ident][key]=value
        return True

    def load_plain_source(self,src,emu,section):
        if src == None:
            return
        self.log("00",_("Initializing ..."))
        ilabel=emu.get("General","identity")
        rx=re.compile(emu.get(section,"pattern"),re.I)
        transform_values=dict()
        for group in rx.groupindex.keys():
            if group in emu.options(section):
                transform_values[group]=emu.get(section,group)
        for line in src:
            match=rx.search(line)
            if match != None:
                ident=match.group(ilabel)
                self.log("00",_("Parsing %s ...") % ident)
                if not self.current.has_key(ident):
                    self.current[ident]=dict()
                    for field in self.config.get("General","fields"):
                        self.current[ident][field]=""
                    self.current[ident][ilabel]=ident
                for key,value in match.groupdict().iteritems():
                    if key != ilabel:
                        self.current[ident][key]=value
        return

    def write(self):
        self.log("10",_("Writing ..."))
        self.log("11",self.config.get("General","gamelist"))
        self.log("00",_(" "))
        signal.signal(signal.SIGTERM,signal.SIG_IGN)
        format = self.config.get("General","gamelist_format")
        if  format == "xml":
            self.write_xml(self.config.get("General","gamelist"))
        elif format == "python":
            pickle.dump(self.list,open(self.config.get("General","gamelist"),"w"))
        else:
            self.log("99",_("Unknown gamelist format ['xml','python'] : %s") % format)
            sys.exit(1)

    def write_xml(self,file):
        fields=self.config.get("General","fields")
        if "emulator" not in fields:
            fields.append("emulator")
        str=''
        for game in self.list:
            str += '  <game'
            for field in fields:
                str += ' %s="%s"' % (field, escape(game[field].encode('utf-8')))
            str += ' />\n'
        fd=open(file,'w')
        gameliststring='<?xml version="1.0" ?>\n<gamelist version="1.0">\n' + str + '</gamelist>\n'
        fd.write(gameliststring)
        return
    def log(self,level,msg):
        self.out.write(level + " " + escape(msg) + "\n")
        self.out.flush()
        return

    def handler(self,signum,frame):
        sys.exit(0)
        return

