
import os
import sys

import gtk

from SimpleGladeApp import SimpleGladeApp
from SimpleGladeApp import bindtextdomain

from Gamelist import Manager
from Config import Configuration
from Preferences import PreferencesDialog
from gobject import TYPE_INT,TYPE_STRING
from zipfile import ZipFile,is_zipfile
from tempfile import mkstemp
from os.path import isdir
from time import sleep
import pango
try:
    from thread import start_new_thread
except ImportError:
    from dummy_thread import start_new_thread

app_name = "Loemu"
app_version = "0.1.1"

glade_dir = sys.prefix + "/share/games/loemu/glade"
if os.access("Loemu.glade",os.R_OK):
    glade_dir=""
locale_dir = sys.prefix + "/share/games/loemu/lang"

bindtextdomain(app_name, locale_dir)

class MainWindow(SimpleGladeApp):

    def __init__(self, path="Loemu.glade",
                 root="main_window",
                 domain=app_name, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

    def new(self):
        
        self.main_window.maximize()
        self.loading_dialog = LoadingDialog()
        self.loading_dialog.loading_dialog.set_transient_for(self.main_window)

        self.conf = Configuration()
        self.conf.load()

        self.gamelist=Manager()

        self.define_select_treeview()
        self.define_select_combobox()
        self.define_games_treeview()
        self.define_show_game_zone()
        self.define_emu_combobox()
        self.find_dialog = FindDialog()
        self.preferences_dialog=PreferencesDialog()

        self.load()

    def define_emu_combobox(self):
        emulators=self.conf.emulators()
        emulators.insert(0,_("All"))
        liststore = gtk.ListStore(TYPE_STRING)
        for emu in emulators:
            liststore.append([emu])
        self.emu_combobox.set_model(liststore)
        cell = gtk.CellRendererText()
        self.emu_combobox.pack_start(cell, True)
        self.emu_combobox.add_attribute(cell, 'text', 0)
        self.emu_combobox.set_active(emulators.index(self.conf.get("General","default_emulator")))
        return

    def define_select_combobox(self):
        labels=self.conf.get("General","labels")
        liststore = gtk.ListStore(TYPE_STRING)
        for select in self.conf.get("General","selects"):
            if select in labels:
                label=labels[select]
            else:
                label=select
            liststore.append([label])
        self.select_combobox.set_model(liststore)
        cell = gtk.CellRendererText()
        self.select_combobox.pack_start(cell, True)
        self.select_combobox.add_attribute(cell, 'text', 0)
        self.last_select_selection = None
        for select,label in labels.iteritems():
            if select == self.conf.get("General","default_select"):
                self.select_combobox.set_active(self.conf.get("General","selects").index(select))
        return

    def define_select_treeview(self):
        self.select_lists=dict()
        for select in self.conf.get("General","selects"):
            self.select_lists[select]=list()
        
        selection=self.select_treeview.get_selection()
        selection.set_mode(gtk.SELECTION_MULTIPLE)
        selection.select_path(0)

        liststore=gtk.ListStore(TYPE_STRING)
        self.select_treeview.set_model(liststore)

        column=gtk.TreeViewColumn()
        self.select_treeview.append_column(column)
        cell = gtk.CellRendererText()
        column.pack_start(cell, True)
        column.set_attributes(cell, text=0)

        self.select_treeview.set_enable_search(False)
        self.select_treeview.set_cursor(0,column)
        return

    def define_games_treeview(self):
        cell = gtk.CellRendererText()
        column=list()
        labels=self.conf.get("General","labels")
        for col,name in enumerate(self.conf.get("General","row_fields")):
            if name in labels:
                label=labels[name]
            else:
                label=name
            column= gtk.TreeViewColumn(label)
            self.games_treeview.append_column(column)
            column.pack_start(cell, True)
            column.set_attributes(cell, text=col+1)
            column.set_sort_column_id(col+1)
            column.set_resizable(True)
            column.set_min_width(50)
        self.games_treeview.set_reorderable(True)
        self.games_treeview.set_enable_search(True)
        self.games_treeview.set_search_column(1)

        selection=self.games_treeview.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.select_path(0)
        return
        
    def define_show_game_zone(self):
        self.game_labels=dict()
        # define the labels that will shown game information
        attrs=pango.AttrList()
        attrs.insert(pango.AttrScale(pango.SCALE_XX_LARGE,0,100))
        attrs.insert(pango.AttrWeight(pango.WEIGHT_BOLD,0,100))
        self.titles_box.pack_start(gtk.Label(),True,True,0)
        for prop in self.conf.get("General","show"):
            self.game_labels[prop]=gtk.Label()
            self.game_labels[prop].set_attributes(attrs)
            self.game_labels[prop].set_alignment(1,0)
            self.titles_box.pack_start(self.game_labels[prop],False,True,0)
        self.titles_box.pack_start(gtk.Label(),True,True,0)
        self.titles_box.show_all()

        self.tempfile=mkstemp()[1]
        self.image_sets=dict()
        return   

    def add_image_sets(self):     
        #
        # assign tuples to every image path.
        #  zip file tuple  => ( ZipFile class , set_image_from_zipfile function )
        #  directory tuple => ( directory     , set_image_from_directory function )
        #
        self.image.set_from_file(self.conf.get("General","default_image"))
        self.image_emu_sets=dict()
        for id in self.conf.emulators():
            self.image_emu_sets[id]=list()
            emu=self.conf.emu(id)
            if emu.has_option("General","images"):
                for image_set in emu.get("General","images"):
                    if image_set not in self.image_sets:
                        if is_zipfile(image_set):
                            self.image_sets[image_set]=(ZipFile(image_set,"r"),self.set_image_from_zipfile)
                        elif isdir(image_set):
                            self.image_sets[image_set]=(image_set+"/",self.set_image_from_directory)
                    self.image_emu_sets[id].append(self.image_sets[image_set])
        return

    def set_image_from_directory(self,filename,directory):
        path=directory + filename
        if not os.access(path,os.R_OK):
            return False
        self.image.set_from_file(path)
        return True
    
    def set_image_from_zipfile(self,filename,zipfile):
        if filename not in zipfile.namelist():
            return False
        data = zipfile.read(filename)
        f = file(self.tempfile,"w")
        f.write(data)
        f.close()
        self.image.set_from_file(self.tempfile)
        return True

    # show current game
    def show_game(self, treeview, *args):
        (model,iter)=treeview.get_selection().get_selected()
        if iter == None:
            return None
        # set label texts
        game=self.gamelist.get(model.get_value(iter,0))
        for prop in self.conf.get("General","show"):
            self.game_labels[prop].set_text(game[prop])
        # set image from image sets
        filename = game["romset"] + ".png"
        id=game["emulator"]
        for image_set,set_image_function in self.image_emu_sets[id]:
            if set_image_function(filename,image_set):
                break
        else:    
            self.image.set_from_file(self.conf.get("General","default_image"))
        return

    def set_games_list(self,match):
        eval_string="TYPE_INT"
        columns=TYPE_STRING,TYPE_STRING,TYPE_STRING
        for i in range(len(self.conf.get("General","row_fields"))):
            eval_string=eval_string+",TYPE_STRING"
        
        liststore = eval("gtk.ListStore(" + eval_string + ")")
        self.games_treeview.set_model(liststore)
        
        selection=self.games_treeview.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.select_path(0)
        
        row_fields=self.conf.get("General","row_fields")
        for index,game in enumerate(match):
            line=list()
            line.append(index)
            for name in row_fields:
                line.append(game[name])
            liststore.append(line)
        
        self.games_treeview.columns_autosize()
        col=self.games_treeview.get_column(1)
        self.games_treeview.set_cursor(0,col)
        self.games_treeview.grab_focus()
        return

    # key_press event
    def key_press(self, widget, key):
        key=gtk.gdk.keyval_name(key.keyval)
        if key == "Right":
            self.filter_selection()
            return True
        elif key == "Left":
            self.select_treeview.grab_focus()
            return True
        return False

    # row_activated event
    def execute_emu(self, *args):
        #self.loading_dialog.open()
        (model,iter)=self.games_treeview.get_selection().get_selected()
        if iter == None:
            return None
        self.gamelist.execute(model.get_value(iter,0))
        #self.loading_dialog.close()
        self.games_treeview.grab_focus()
        return

    # filter the selection
    def filter_selection(self, widget=None, *args):
        (model,paths)=self.select_treeview.get_selection().get_selected_rows()
        if paths == self.last_select_selection:
            self.games_treeview.grab_focus()
            return None
        self.last_select_selection= paths

        selection=list()
        for selec in paths:
            iter=model.get_iter(selec[0])
            selection.append(model.get_value(iter,0))

        self.gamelist.set_selection(self.current_select,selection)
        match=self.gamelist.filter()
        self.set_games_list(match)
        return

    def change_emulator(self, widget, *args):
        emulator=self.emu_combobox.get_active_text()
        self.gamelist.set_emulator(emulator)
        (model,paths)=self.games_treeview.get_selection().get_selected_rows()
        if model == None:
            return
        match=self.gamelist.filter()
        self.set_games_list(match)
        return

    # change category type
    def change_select(self, combobox, *args):
        active_text=self.select_combobox.get_active_text()
        labels=self.conf.get("General","labels")
        if active_text in labels.values():
            for select,label in labels.iteritems():
                if label == active_text:
                    self.current_select=select
        else:
            self.current_select=active_text
        
        liststore=self.select_treeview.get_model()
        liststore.clear() 
        for row in self.select_lists[self.current_select]:
            liststore.append([row])
        
        col=self.select_treeview.get_column(0)
        self.select_treeview.set_cursor(0,col)
        self.select_treeview.grab_focus()
        self.last_select_selection = None
        return

    # open the find dialog and filter the pattern
    def open_find_dialog(self, *args):
        response = self.find_dialog.open()
        if response != gtk.RESPONSE_OK and response != gtk.RESPONSE_APPLY:
            self.find_dialog.close()
            return
        pattern=self.find_dialog.get_values()
        match=self.gamelist.filter(pattern)
        self.set_games_list(match)
        self.last_select_selection = None
        if response == gtk.RESPONSE_APPLY:
            self.open_find_dialog()
        self.find_dialog.close()
        return

    def open_preferences_dialog(self, widget, *args):
        self.preferences_dialog.open()
        response=gtk.RESPONSE_APPLY
        while response == gtk.RESPONSE_APPLY:
            response = self.preferences_dialog.preferences_dialog.run()
            if response in [gtk.RESPONSE_OK,gtk.RESPONSE_APPLY]:
                self.preferences_dialog.update_configuration()
        self.preferences_dialog.close()
        while gtk.events_pending():
            gtk.main_iteration_do(False)
        self.conf.load()

        self.load()
        return

    def load(self):
        self.loading_dialog.loading_dialog.show()
        while gtk.events_pending():
            gtk.main_iteration_do(False)
        self.loading_in_progress=True
        start_new_thread(self.load_process,())
        while self.loading_in_progress:
            self.loading_dialog.progressbar.pulse()
            while gtk.events_pending():
                gtk.main_iteration_do(False)
            sleep(0.2)
        self.loading_dialog.loading_dialog.hide()
        return

    def load_process(self,*args):
        self.add_image_sets()
        if not self.preferences_dialog.gamelist_is_rebuilded():
            self.loading_in_progress=False
            return
        if not os.access(self.conf.get("General","gamelist"),os.R_OK):
            self.loading_in_progress=False
            return
        err = self.gamelist.load()
        if err:
            dlg=gtk.MessageDialog(self.main_window, gtk.DIALOG_MODAL,gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,err)
            dlg.run()
            dlg.destroy()
        self.select_lists=self.gamelist.get_filter_lists()
        self.change_select(self.select_combobox)
        self.loading_in_progress=False
        return

class LoadingDialog(SimpleGladeApp):

    def __init__(self, path="Loemu.glade",
                 root="loading_dialog",
                 domain=app_name, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

class FindDialog(SimpleGladeApp):

    def __init__(self, path="Loemu.glade",
                 root="find_dialog",
                 domain=app_name, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

    def new(self):
        self.conf = Configuration()
        self.conf.load()
        labels=self.conf.get("General","labels")
        self.filter= dict()
        for entry in self.conf.get("General","filters"):
            if entry not in labels:
                labels[entry]=entry
            label=gtk.Label(labels[entry])
            label.set_alignment(1,0)
            self.titles_box.pack_start(label, False, True, 6)
        
            self.filter[entry]=gtk.Entry()
            self.filter[entry].set_max_length(50)
            self.filter[entry].select_region(0, len(self.filter[entry].get_text()))
            self.entries_box.pack_start(self.filter[entry], False, True, 1)
        self.entries_box.show_all()
        self.titles_box.show_all()
    
    def open(self):
        self.find_dialog.show()
        return self.find_dialog.run()
    
    def close(self):
        self.find_dialog.hide()
        return

    def get_values(self):
        pattern=dict()
        for entry in self.filter:
            pattern[entry]=self.filter[entry].get_text()
        return pattern

def main():
    main_window = MainWindow()

    main_window.run()

if __name__ == "__main__":
    main()

