
import os
import sys

import gtk

from SimpleGladeApp import SimpleGladeApp
from SimpleGladeApp import bindtextdomain

from Config import Configuration
from Gamelist import Builder
from gobject import TYPE_STRING
from time import sleep
from tempfile import mkstemp
from subprocess import Popen,PIPE
from signal import SIGTERM

try:
    from thread import start_new_thread
except ImportError:
    from dummy_thread import start_new_thread

app_name = "Loemu"
app_version = "0.1.1"

glade_dir = sys.prefix + "/share/games/loemu/glade"
if os.access("Preferences.glade",os.R_OK):
    glade_dir=""
locale_dir = sys.prefix + "/share/games/loemu/lang"

bindtextdomain(app_name, locale_dir)


class PreferencesDialog(SimpleGladeApp):

    def __init__(self, path="Preferences.glade",
                 root="preferences_dialog",
                 domain=app_name, **kwargs):
        if not os.access(path,os.R_OK):
            path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

    def new(self):
        self.conf = Configuration()
        self.conf.load()
        self.set_general_page_variables()

        self.pages=dict()
        num_emus=self.notebook.get_n_pages()
        for num in range(1,num_emus):
            page = self.notebook.get_nth_page(num)
            label = self.notebook.get_tab_label_text(page)
            self.pages[label] = page

        for id in self.conf.emulators():
            emu=self.conf.emu(id)
            if id in self.pages:
                self.set_page_variables(emu)
            else:
                if emu.has_section('UserDependent'):
                    self.set_page(emu)

        self.about_dialog = AboutDialog()
        self.about_dialog.about_dialog.set_transient_for(self.preferences_dialog)
        self.about_dialog.about_dialog.set_modal(True)
        self.building_dialog = BuildingDialog()
        self.building_dialog.building_dialog.set_transient_for(self.preferences_dialog)
        self.gamelist_rebuilded=True

    def set_general_page_variables(self):
        self.rlabels=dict()
        labels=self.conf.get("General","labels")
        for tag,label in labels.iteritems():
            self.rlabels[label]=tag
        self.set_general_checkbuttons("shown_states")
        
        emulators=self.conf.emulators()
        emulators.insert(0,_("All"))
        self.set_general_combobox("default_emulator",emulators)
        self.set_general_combobox("default_select",self.conf.get("General","selects"))
        self.set_general_combobox("gamelist_format",['xml','python'])

        self.gamelist.set_filename(self.conf.get('UserDependent',"gamelist"))
        filename=self.conf.get('UserDependent',"default_image")
        if os.access(filename,os.R_OK):
            self.default_image.set_filename(filename)

    def set_general_combobox(self,key,list):
        widget=self.get_widget(key)

        liststore = gtk.ListStore(TYPE_STRING)
        for item in list:
            liststore.append([item])
        widget.set_model(liststore)
        cell = gtk.CellRendererText()
        widget.pack_start(cell, True)
        widget.add_attribute(cell, 'text', 0)
        default=self.conf.get('UserDependent',key)
        match=self.rlabels.get(default,default)
        
        widget.set_active(list.index(match))
        return

    def set_general_checkbuttons(self,key):
        widgets=self.glade.get_widget_prefix(key)
        match=self.conf.get('UserDependent',key)
        for widget in widgets:
            label=widget.get_label()
            if self.rlabels.get(label,label) in match:
                widget.set_active(True)
        return

    def set_page_variables(self,emu):
        if not emu.has_section('UserDependent'):
            return
        id=emu.get("General","label")
        page=self.pages[id]
        for key in emu.options('UserDependent'):
            widget=self.find_child_by_name(page,key)
            if isinstance(widget,gtk.TreeView):
                self.set_treeview(widget,emu.get('UserDependent',key))
            if isinstance(widget,gtk.FileChooserButton):
                value=emu.get('UserDependent',key)
                if os.access(value,os.R_OK):
                    widget.set_filename(value)
            if isinstance(widget,gtk.Entry):
                value=emu.get('UserDependent',key)
                widget.set_text(str(value))
        return

    def set_treeview(self,treeview,list):
        liststore=gtk.ListStore(TYPE_STRING)
        treeview.set_model(liststore)
        
        column=gtk.TreeViewColumn()
        treeview.append_column(column)
        cell = gtk.CellRendererText()
        column.pack_start(cell, True)
        column.set_attributes(cell, text=0)

        treeview.set_enable_search(False)
        for row in list:
            liststore.append([row])
    
    # this function is executed to define the preferences page
    # of a new emulator that not yet exists
    def set_page(self,emu):
        box = gtk.VBox(False,10)
        box.set_border_width(10)
        id=emu.get("General","label")
        self.notebook.append_page(box,gtk.Label(id))

        self.pages[id] = box

        label = gtk.Label()
        label.set_markup('<big><b>'+id+' User Configuration</b></big>')
        box.pack_start(label,False,True,10)

        hbox = gtk.HBox(False,5)
        box.pack_start(hbox)
        
        label_box = gtk.VBox(False,0)
        entry_box = gtk.VBox(False,0)
        hbox.pack_start(label_box,False,True)
        hbox.pack_start(entry_box,True,True)
        
        for key,value in emu.items('UserDependent'):
            if not self.get_widget(id+'_'+key): 
                label = gtk.Label(key)
                label.set_alignment(0,1)
                label_box.pack_start(label, False, True, 6)
                
                entry = gtk.Entry()
                entry.set_name(key)
                entry.set_text(str(value))
                entry_box.pack_start(entry, False, True, 1)

        build_box=gtk.HBox(False,15)
        build_box.set_border_width(20)
        label=gtk.Label()
        label.set_markup("<b>"+_("Build partial Gamelist")+"</b>")
        build_box.pack_start(label,False)

        button = gtk.Button(stock=gtk.STOCK_EXECUTE)
        button.connect("clicked", self.build_gamelist, None)
        alignment = button.get_children()[0]
        child_box = alignment.get_children()[0]
        child_box.set_border_width(5)
        image, label = child_box.get_children()
        label.set_text(_('Build Gamelist'))

        button_box=gtk.HBox(False, 10)
        button_box.pack_start(button,False)
        build_box.pack_end(button_box)

        box.pack_end(build_box,False,True,10)
        box.show_all()
        return
        
    def update_configuration(self):
        page=self.notebook.get_nth_page(0)
        widget=self.find_child_by_name(page,"default_image")
        filename = widget.get_filename()
        if filename == None:
            filename = ""
        self.conf.set('UserDependent',"default_image",filename)
        widget=self.find_child_by_name(page,"default_select")
        self.conf.set('UserDependent',"default_select",widget.get_active_text())
        widget=self.find_child_by_name(page,"default_emulator")
        self.conf.set('UserDependent',"default_emulator",widget.get_active_text())
        widget=self.find_child_by_name(page,"gamelist")
        self.conf.set('UserDependent',"gamelist",widget.get_filename())
        widget=self.find_child_by_name(page,"gamelist_format")
        self.conf.set('UserDependent',"gamelist_format",widget.get_active_text())
        widgets=self.glade.get_widget_prefix("shown_states")
        values=list()
        for widget in widgets:
            if widget.get_active():
                values.append(self.rlabels[widget.get_label()])
        self.conf.set('UserDependent',"shown_states",repr(values))

        for id,page in self.pages.iteritems(): 
            emu=self.conf.emu(id)
            for key in emu.options('UserDependent'):
                widget=self.find_child_by_name(page,key)
                if isinstance(widget,gtk.TreeView):
                    liststore=widget.get_model()
                    values=list()
                    iter=liststore.get_iter_first()
                    while iter:
                        values.append(liststore.get_value(iter,0))
                        iter=liststore.iter_next(iter)
                    emu.set('UserDependent',key,repr(values))
                if isinstance(widget,gtk.FileChooserButton):
                    filename = widget.get_filename()
                    if filename == None:
                        filename = ""
                    emu.set('UserDependent',key,filename)
                if isinstance(widget,gtk.Entry):
                    emu.set('UserDependent',key,widget.get_text())
        self.conf.write_configuration()
        return

    def open(self):
        self.gamelist_rebuilded = False
        self.preferences_dialog.show()
        return 
    
    def close(self):
        self.preferences_dialog.hide()
        return

    def gamelist_is_rebuilded(self):
        return self.gamelist_rebuilded

    def get_page_id(self,widget):
        id = self.notebook.get_tab_label_text(widget)
        if id != None:
            return id
        return self.get_page_id(widget.get_parent())

    def find_child_by_name(self,widget,name):
        try:
            children=widget.get_children()
        except AttributeError:
            return None
        for child in children:
            if child.get_name() == name:
                return child
            else:
                rchild = self.find_child_by_name(child,name)
                if rchild != None:
                    return rchild
        return None

    def update_image(self, widget, *args):
        filename = widget.get_filename()
        if filename == None:
            return
        try:
            pixbuf=gtk.gdk.pixbuf_new_from_file_at_size(filename,70,70)
        except:
            return
        self.image.set_from_pixbuf(pixbuf)

    def open_about_dialog(self, widget, *args):
        self.about_dialog.about_dialog.show()
        self.about_dialog.about_dialog.run()
        self.about_dialog.about_dialog.hide()
        return

    def treeview_images_action(self, src_widget, *args):
        name=src_widget.get_name()
        for action in ['file','directory','remove','up','down']:
            if action in name:
                break;
        else:
            return
        id=self.get_page_id(src_widget)
        treeview=self.find_child_by_name(self.pages[id],"images")
        liststore=treeview.get_model()
        if action == "file":
            chooser = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_OPEN,buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
            if chooser.run() == gtk.RESPONSE_ACCEPT:
                liststore.append([chooser.get_filename()])
            chooser.destroy()
            return
        if action == "directory":
            chooser = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
            if chooser.run() == gtk.RESPONSE_ACCEPT:
                liststore.append([chooser.get_current_folder()])
            chooser.destroy()
            return
        
        (model,iter)=treeview.get_selection().get_selected()
        if iter==None:
            return
        
        if action == "remove":
            liststore.remove(iter)
            return
        if action == "up":
            num=liststore.get_path(iter)[0]
            if num==0:
                return
            prev_iter=liststore.get_iter(num-1)
            liststore.swap(iter,prev_iter)
            return
        if action == "down":
            next_iter=liststore.iter_next(iter)
            if next_iter==None:
                return
            liststore.swap(iter,next_iter)
            return
        return

    def build_gamelist(self, widget, *args):
        self.update_configuration()
        dlg=self.building_dialog.build(self.get_page_id(widget))
        dlg.set_transient_for(self.preferences_dialog)
        dlg.run()
        dlg.destroy()
        self.gamelist_rebuilded=True
        return

class BuildingDialog(SimpleGladeApp):

    def __init__(self, path="Preferences.glade",
                 root="building_dialog",
                 domain=app_name, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

    def build(self,id):
        self.title.set_markup("<b>"+_("Reading configuration ...")+"</b>")
        self.subtitle.set_text(" ")
        self.msg.set_text(" ")
        self.building_dialog.show()
        while gtk.events_pending():
            gtk.main_iteration_do(False)

        self.msg_dialog=gtk.MessageDialog(  flags = gtk.DIALOG_MODAL,
                                            type = gtk.MESSAGE_INFO,
                                            buttons = gtk.BUTTONS_OK)
        self.msg_dialog.set_markup("<b>"+_("Process completed!")+"</b>")

        args=list()
        args.append("loemu-builder")
        if id != "General":
            args.append(id)
        process_environ=os.environ.copy()
        process_environ['PATH']=os.curdir+":"+process_environ['PATH']

        p=Popen(args,stdout=PIPE,env=process_environ)
        self.pid=p.pid
        self.process_is_running=True
        start_new_thread(self.builder_log_handler,(p.stdout,))

        while self.process_is_running:
            self.progressbar.pulse()
            while gtk.events_pending():
                gtk.main_iteration_do(False)
            sleep(0.5)
        self.building_dialog.hide()
        return self.msg_dialog

    def builder_log_handler(self,stdout,*args):
        line=stdout.readline()
        while line != "":
            if line[0:2]=="00":
                self.msg.set_markup("<i>"+line[3:-1]+"</i>")
            elif line[0:2]=="10":
                self.title.set_markup("<b>"+line[3:-1]+"</b>")
            elif line[0:2]=="11":
                self.subtitle.set_text(line[3:-1])
            elif line[0:2]=="99":
                self.msg_dialog=gtk.MessageDialog(  flags = gtk.DIALOG_MODAL,
                                                    type = gtk.MESSAGE_ERROR,
                                                    buttons = gtk.BUTTONS_OK)
                self.msg_dialog.set_markup("<b>"+_("Building canceled!")+"</b>")
                self.msg_dialog.format_secondary_text(line[3:-1])
            line=stdout.readline()
        self.process_is_running = False
        return

    def cancel(self, widget, *args):
        os.kill(self.pid,SIGTERM)
        self.msg_dialog=gtk.MessageDialog(  flags = gtk.DIALOG_MODAL,
                                            type = gtk.MESSAGE_ERROR,
                                            buttons = gtk.BUTTONS_OK)
        self.msg_dialog.set_markup("<b>"+_("Building canceled!")+"</b>")
        return True

class AboutDialog(SimpleGladeApp):

    def __init__(self, path="Preferences.glade",
                 root="about_dialog",
                 domain=app_name, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

def main():
    preferences_dialog = PreferencesDialog()
    preferences_dialog.open()
    response=gtk.RESPONSE_APPLY
    while response == gtk.RESPONSE_APPLY:
        response = preferences_dialog.preferences_dialog.run()
        if response in [gtk.RESPONSE_OK,gtk.RESPONSE_APPLY]:
            preferences_dialog.update_configuration()
    preferences_dialog.close()

if __name__ == "__main__":
    main()
